@@ -491,7 +491,7 @@ class NamelistName(SortedDict):
         key = self._keyCaseInsensitive(key)
 
         # handle internal key indexing
-        if '(' in key and re.match(r'.*\(.*\)', key) and key not in self.keys():
+        if isinstance(key, str) and '(' in key and re.match(r'.*\(.*\)', key) and key not in self.keys():
             index = np.array(list(map(int, re.sub(r'(.*)\((.*)\)', r'\2', key).split(','))))
             key = re.sub(r'(.*)\((.*)\)', r'\1', key)
         else:
