@@ -1182,7 +1182,7 @@ class OMFITmhdin(OMFITnamelist):
         """
 
         geqdsk = OMFITgeqdsk('geqdsk')
-        geqdsk['CASE'] = ['1', '2', '3', '4', '5', '6']
+        geqdsk['CASE'] = ['1', '2', '3', '#999999', '1000ms', '6']
         geqdsk['NW'] = nw = self['IN3']['NW']
         geqdsk['NH'] = nh = self['IN3']['NH']
         geqdsk['RDIM'] = self['IN3']['RRIGHT'] - self['IN3']['RLEFT']
