@@ -130,10 +130,8 @@ class OMFITjson(SortedDict, OMFITascii):
     @dynaSave
     def save(self):
         with open(self.filename, 'w') as f:
-            try:
-                output = json.dumps(self, default=lambda x: dumper(x, self.objects_encode), indent=1, separators=(',', ': '))
-            except AttributeError as _excp:
-                raise AttributeError(str(_excp) + ':\nMake sure that the dictionary entries ')
+            output = json.dumps(self, default=lambda x: dumper(x, self.objects_encode), indent=1, separators=(',', ': '))
+
             if self.use_leading_comma:
                 output = re.sub(r",\n(\s+)", r"\n\1,", output)
             f.write(output + '\n')
@@ -151,6 +149,11 @@ class OMFITjson(SortedDict, OMFITascii):
             tmp.pop('objects_encode')
         return tmp
 
+    def __setitem__(self, key, value):
+        if isinstance(key, (np.int64, np.int32)):
+            key = int(key)
+        return super().__setitem__(key, value)
+
 
 class SettingsName(SortedDict):
     """
@@ -160,10 +163,14 @@ class SettingsName(SortedDict):
     baseDict = None
 
     def __setitem__(self, key, value):
+        if isinstance(key, (np.int64, np.int32)):
+            key = int(key)
+
         if isinstance(value, dict) and not isinstance(value, self.baseDict):
             tmp = self.baseDict()
             tmp.update(value)
             value = tmp
+
         return super().__setitem__(key, value)
 
     def __repr__(self):
