@@ -332,6 +332,9 @@ class OMFITmars(SortedDict, OMFITobject):
             ('omega_*e', 'e- dia. drift freq.', None),
             ('CSV', 'CSV', None),
             ('erot_prof', 'electron rotation', None),
+            ('nu_i', 'ion eff. collisionality', None),
+            ('nu_e', 'e- eff. collisionality', None),
+            ('zeff', 'Z effective', None),
         ]
 
         x = data[:, 0]
@@ -614,10 +617,11 @@ class OMFITmars(SortedDict, OMFITobject):
         else:
             return RZrw_EF
 
-    def get_UnitVec(self, vacFlag=False):
+    def get_UnitVec(self, vacFlag=False, IIgrid=None):
         """
         Get unit vectors e_s and e_chi and jacobian from real space R,Z
         :param vacFlag: flag to calculate metric elements in all domain (plasma+vacuum)
+        :param IIgrid: specify the radial mesh index to calculate vectors within
         """
         from scipy.interpolate import InterpolatedUnivariateSpline
 
@@ -636,6 +640,10 @@ class OMFITmars(SortedDict, OMFITobject):
         Ns2 = self[self.sim]['Ns2'].values.astype(int)
         # indexes of full domain: first couple is vacuum, second is plasma
         domain = [(Ns1, Ns1 + Ns2), (0, Ns1)]
+        if isinstance(IIgrid, int):
+            II = IIgrid
+        else:
+            II = Ns1
         # Loop over vacuum and plasma
         for ii, kk in domain:
             if ii == Ns1:
@@ -709,11 +717,11 @@ class OMFITmars(SortedDict, OMFITobject):
             G33 = np.square(self[self.sim]['R'].values)
         else:
             # Calculate metric elements inside the plasma
-            G11 = np.square(dRds[0:Ns1, :]) + np.square(dZds[0:Ns1, :])
-            G12 = np.multiply(dRds[0:Ns1, :], dRdchi[0:Ns1, :]) + np.multiply(dZds[0:Ns1, :], dZdchi[0:Ns1, :])
-            G22 = np.square(dRdchi[0:Ns1, :]) + np.square(dZdchi[0:Ns1, :])
+            G11 = np.square(dRds[0:II, :]) + np.square(dZds[0:II, :])
+            G12 = np.multiply(dRds[0:II, :], dRdchi[0:II, :]) + np.multiply(dZds[0:II, :], dZdchi[0:II, :])
+            G22 = np.square(dRdchi[0:II, :]) + np.square(dZdchi[0:II, :])
             G22[0, :] = G22[1, :]
-            G33 = np.square(self[self.sim]['R'].values[0:Ns1, :])
+            G33 = np.square(self[self.sim]['R'].values[0:II, :])
         # Store data
         self['UnitVector'] = xarray.Dataset()
         self['UnitVector']['dRds'] = xarray.DataArray(
@@ -746,18 +754,17 @@ class OMFITmars(SortedDict, OMFITobject):
                 G33, dims=['s', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 's': self[self.sim]['s']}
             )
         else:
-            # Radial coordinate name 'sp' is within plasma
             self['Metric']['G11'] = xarray.DataArray(
-                G11, dims=['sp', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 'sp': self[self.sim]['sp']}
+                G11, dims=['s', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 's': self[self.sim]['s'][0:II]}
             )
             self['Metric']['G12'] = xarray.DataArray(
-                G12, dims=['sp', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 'sp': self[self.sim]['sp']}
+                G12, dims=['s', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 's': self[self.sim]['s'][0:II]}
             )
             self['Metric']['G22'] = xarray.DataArray(
-                G22, dims=['sp', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 'sp': self[self.sim]['sp']}
+                G22, dims=['s', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 's': self[self.sim]['s'][0:II]}
             )
             self['Metric']['G33'] = xarray.DataArray(
-                G33, dims=['sp', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 'sp': self[self.sim]['sp']}
+                G33, dims=['s', 'chi'], coords={'chi': np.linspace(np.pi * -1, np.pi, Nchi), 's': self[self.sim]['s'][0:II]}
             )
         return dRds, dZds, dRdchi, dZdchi, jacobian
 
@@ -828,45 +835,73 @@ class OMFITmars(SortedDict, OMFITobject):
         Inverse Fourier transorm of perturbed magnetic field and calculate Bn
         """
         if 'UnitVector' not in self:
-            self.get_UnitVec(vacFlag=True)
+            self.get_UnitVec()
+        # Check metric element size
+        # This is needed when IIgrid=None and NV(MARS) != NV(CHEASE)
+        II = np.size(self['BPLASMA']['BM1'], 0)
+        if np.size(self['Metric']['G22'], 0) != II:
+            print("WARNING: NV(MARS) != NV(CHEASE). Calculating metric elements with b-field radial grid size.")
+            self.get_UnitVec(IIgrid=II)
+        # Define DFT matrices
         expmchi = np.exp(self['BPLASMA']['Mm'] * self[self.sim]['chi'] * 1j)
         expmchi_inv = np.exp(-self[self.sim]['chi'] * self['BPLASMA']['Mm'] * 1j)
         chione = np.ones((1, len(self[self.sim]['chi'])))
-        Ns1 = self[self.sim]['Ns1'].values
         # Calculate inverse Fourier transorm
-        B1 = np.dot(self['BPLASMA']['BM1'].values, expmchi)
-        B2 = np.dot(self['BPLASMA']['BM2'].values, expmchi)
-        B3 = np.dot(self['BPLASMA']['BM3'].values, expmchi)
-        # Check metric element definition
-        if np.size(self['Metric']['G22'], 0) != np.size(B1, 0):
-            self.get_UnitVec(vacFlag=True)
+        B1 = np.dot(self['BPLASMA']['BM1'].isel(s=range(II)), expmchi)
+        B2 = np.dot(self['BPLASMA']['BM2'].isel(s=range(II)), expmchi)
+        B3 = np.dot(self['BPLASMA']['BM3'].isel(s=range(II)), expmchi)
         # Calculate normal perturbeb magnetic field
-        Bn = np.divide(B1, np.divide(np.sqrt(self['Metric']['G22']), self[self.sim]['R']))
+        Bn = np.divide(B1, np.multiply(np.sqrt(self['Metric']['G22']), self[self.sim]['R'].isel(s=range(II))))
         # Fourier transorm of Bn
         BnM = np.dot(Bn, expmchi_inv) * (self[self.sim]['chi'].values[1] - self[self.sim]['chi'].values[0]) / (2 * np.pi)
         # Store data
-        self['BPLASMA']['B1'] = xarray.DataArray(B1, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['B2'] = xarray.DataArray(B2, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['B3'] = xarray.DataArray(B3, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['Bn'] = xarray.DataArray(Bn, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['BnM'] = xarray.DataArray(BnM, dims=['s', 'Mm'], coords={'s': self[self.sim]['s'], 'Mm': self['BPLASMA']['Mm']})
+        self['BPLASMA']['B1'] = xarray.DataArray(B1, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['B2'] = xarray.DataArray(B2, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['B3'] = xarray.DataArray(B3, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['Bn'] = xarray.DataArray(Bn, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['BnM'] = xarray.DataArray(BnM, dims=['s', 'Mm'], coords={'s': self['BPLASMA']['s'], 'Mm': self['BPLASMA']['Mm']})
+
+    def get_J123(self):
+        """
+        Inverse Fourier transorm of perturbed current
+        """
+        if 'UnitVector' not in self:
+            self.get_UnitVec()
+        # Check metric element size
+        # This is needed when IIgrid=None and NV(MARS) != NV(CHEASE)
+        II = np.size(self['JPLASMA']['JM1'], 0)
+        if np.size(self['Metric']['G22'], 0) != II:
+            print("WARNING: NV(MARS) != NV(CHEASE). Calculating metric elements with b-field radial grid size.")
+            self.get_UnitVec(IIgrid=II)
+        # Define DFT matrices
+        expmchi = np.exp(self['JPLASMA']['Mm'] * self[self.sim]['chi'] * 1j)
+        expmchi_inv = np.exp(-self[self.sim]['chi'] * self['JPLASMA']['Mm'] * 1j)
+        chione = np.ones((1, len(self[self.sim]['chi'])))
+        # Calculate inverse Fourier transorm
+        J1 = np.dot(self['JPLASMA']['JM1'].isel(s=range(II)), expmchi)
+        J2 = np.dot(self['JPLASMA']['JM2'].isel(s=range(II)), expmchi)
+        J3 = np.dot(self['JPLASMA']['JM3'].isel(s=range(II)), expmchi)
+        # Store data
+        self['JPLASMA']['J1'] = xarray.DataArray(J1, dims=['s', 'chi'], coords={'s': self['JPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['JPLASMA']['J2'] = xarray.DataArray(J2, dims=['s', 'chi'], coords={'s': self['JPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['JPLASMA']['J3'] = xarray.DataArray(J3, dims=['s', 'chi'], coords={'s': self['JPLASMA']['s'], 'chi': self[self.sim]['chi']})
 
     def get_Bcyl(self):
         """
         Get B-field components in cylindrical coordinates
         """
+        print('GET: calculating B-field cylindrical components')
         if 'B1' not in self['BPLASMA']:
             self.get_B123()
-        if 'UnitVector' not in self:
-            self.get_UnitVec(vacFlag=True)
         if 'R' not in self[self.sim]:
             self.get_RZ()
-        J = self['UnitVector']['Jacobian']
-        dRds = self['UnitVector']['dRds']
-        dRdchi = self['UnitVector']['dRdchi']
-        dZds = self['UnitVector']['dZds']
-        dZdchi = self['UnitVector']['dZdchi']
-        R0 = self[self.sim]['R']
+        II = np.size(self['BPLASMA']['BM1'], 0)
+        J = self['UnitVector']['Jacobian'].isel(s=range(II))
+        dRds = self['UnitVector']['dRds'].isel(s=range(II))
+        dRdchi = self['UnitVector']['dRdchi'].isel(s=range(II))
+        dZds = self['UnitVector']['dZds'].isel(s=range(II))
+        dZdchi = self['UnitVector']['dZdchi'].isel(s=range(II))
+        R0 = self[self.sim]['R'].isel(s=range(II))
         # Calculate Br,z,phi
         Br = (self['BPLASMA']['B1'] * dRds + self['BPLASMA']['B2'] * dRdchi) / J
         Br[0:2, :] = Br[2, :]
@@ -874,9 +909,40 @@ class OMFITmars(SortedDict, OMFITobject):
         Bz[0:2, :] = Bz[2, :]
         Bphi = (self['BPLASMA']['B3'] * R0) / J
         Bphi[0:2, :] = Bphi[2, :]
-        self['BPLASMA']['Br'] = xarray.DataArray(Br, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['Bz'] = xarray.DataArray(Bz, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
-        self['BPLASMA']['Bphi'] = xarray.DataArray(Bphi, dims=['s', 'chi'], coords={'s': self[self.sim]['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['Br'] = xarray.DataArray(Br, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['Bz'] = xarray.DataArray(Bz, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']})
+        self['BPLASMA']['Bphi'] = xarray.DataArray(
+            Bphi, dims=['s', 'chi'], coords={'s': self['BPLASMA']['s'], 'chi': self[self.sim]['chi']}
+        )
+
+    def get_Xcyl(self):
+        """
+        Get displacement components in cylindrical coordinates
+        """
+        print('GET: calculating plasma displacement cylindrical components')
+        if 'X1' not in self['XPLASMA']:
+            self.get_X123()
+        if 'R' not in self[self.sim]:
+            self.get_RZ()
+        II = np.size(self['XPLASMA']['XM1'], 0)
+        J = self['UnitVector']['Jacobian'].isel(s=range(II))
+        dRds = self['UnitVector']['dRds'].isel(s=range(II))
+        dRdchi = self['UnitVector']['dRdchi'].isel(s=range(II))
+        dZds = self['UnitVector']['dZds'].isel(s=range(II))
+        dZdchi = self['UnitVector']['dZdchi'].isel(s=range(II))
+        R0 = self[self.sim]['R'].isel(s=range(II))
+        # Calculate Xr,z,phi
+        Xr = (self['XPLASMA']['X1'].values * dRds.values + self['XPLASMA']['X2'].values * dRdchi.values) / J.values
+        Xr[0:2, :] = Xr[2, :]
+        Xz = (self['XPLASMA']['X1'].values * dZds.values + self['XPLASMA']['X2'].values * dZdchi.values) / J.values
+        Xz[0:2, :] = Xz[2, :]
+        Xphi = (self['XPLASMA']['X3'].values * R0.values) / J.values
+        Xphi[0:2, :] = Xphi[2, :]
+        self['XPLASMA']['Xr'] = xarray.DataArray(Xr, dims=['sp', 'chi'], coords={'sp': self['XPLASMA']['sp'], 'chi': self[self.sim]['chi']})
+        self['XPLASMA']['Xz'] = xarray.DataArray(Xz, dims=['sp', 'chi'], coords={'sp': self['XPLASMA']['sp'], 'chi': self[self.sim]['chi']})
+        self['XPLASMA']['Xphi'] = xarray.DataArray(
+            Xphi, dims=['sp', 'chi'], coords={'sp': self['XPLASMA']['sp'], 'chi': self[self.sim]['chi']}
+        )
 
     def get_dWk(self, NFIT=3):
         """
@@ -884,6 +950,7 @@ class OMFITmars(SortedDict, OMFITobject):
 
         :param NFIT: integral of energy density skips the first NFIT+1 points
         """
+        print('GET: calculating dWk components')
         if 'DWK_ENERGY_DENSITY' not in self:
             self.read_dWk_den()
         den = self['DWK_ENERGY_DENSITY']['FULL_MATRIX']
@@ -971,6 +1038,119 @@ class OMFITmars(SortedDict, OMFITobject):
                 self['DWK_COMPONENTS']['K_%s_NTB' % particle] = K_NTB
                 self['DWK_COMPONENTS']['K_%s_NP' % particle] = K_NP
 
+    def get_BatSurf(self, rsurf, kdR=False):
+        """
+        Calculate normal and tangential components of BPLASMA on given surface (e.g. wall)
+        Calculate Fourier decomposition of Bn,t,phi
+        Method used in CarMa Forward Coupling
+        :param rsurf: normalized radius of surface
+        :param kdR: flag for alternative calculation of unit vectors (default=False)
+        """
+        print('GET: calculating B-field on r=%s surface' % (rsurf))
+        if 'B1' not in self['BPLASMA']:
+            self.get_B123()
+        if 'R' not in self[self.sim]:
+            self.get_RZ()
+        if 'Br' not in self['BPLASMA']:
+            self.get_Bcyl()
+        mm = self['BPLASMA']['Mm'].values
+
+        # Find wall surface index
+        IIsurf = np.argmin(np.abs(self[self.sim]['s'].values - rsurf))
+
+        Rw = self[self.sim]['R'].isel(s=IIsurf)
+        Zw = self[self.sim]['Z'].isel(s=IIsurf)
+        Cw = self[self.sim]['chi'].values
+
+        R = self[self.sim]['R']
+        Z = self[self.sim]['Z']
+
+        Tw = np.arctan2(Zw - Z[0, 0], Rw - R[0, 0])
+        KK = np.where(np.diff(Tw) < -np.pi)[0]
+        if (len(KK) == 1) & (2 * KK > len(Tw)):
+            Tw[KK[0] + 1 :] = Tw[KK[0] + 1 :] + 2 * np.pi
+        if (len(KK) == 1) & (2 * KK < len(Tw)):
+            Tw[0 : KK[0]] = Tw[0 : KK[0]] - 2 * np.pi
+        Tw = np.sort(Tw)
+        JJ = np.argsort(Tw)
+        try:
+            Bwr = self['BPLASMA']['Br'].isel(s=IIsurf)
+            Bwz = self['BPLASMA']['Bz'].isel(s=IIsurf)
+            Bwphi = self['BPLASMA']['Bphi'].isel(s=IIsurf)
+        except IndexError:
+            print('ERROR: Selected index is out of BPLASMA calculation domain')
+            return
+
+        # clean Tw,Rw,Zw,Bwr,Bwz,Bwphi
+        [Tw, JJ] = np.unique(Tw, return_index=True)
+        Rw = Rw[JJ]
+        Zw = Zw[JJ]
+        Cw = Cw[JJ]
+        Bwr = Bwr[JJ]
+        Bwz = Bwz[JJ]
+        Bwphi = Bwphi[JJ]
+        # get Gauss quadrature points for Fourier decomposition
+        z, w = np.polynomial.legendre.leggauss(2)
+        x0 = (Tw[:-1] + Tw[1:]) * 0.5
+        h2 = np.diff(Tw) * 0.5
+        xx = z[:, np.newaxis] * h2[np.newaxis, :] + np.ones(z.shape)[:, np.newaxis] * x0[np.newaxis, :]
+        wh = w[:, np.newaxis] * h2[np.newaxis, :]
+        xx = xx.reshape(-1, 1)
+        wh = wh.reshape(-1, 1)
+
+        if kdR == True:
+            # Fourier decompose (Rw,Zw) in Tw-angle
+            expmt = np.exp(-1j * xx * mm) / (2 * np.pi)
+
+            Rx = scipy.interpolate.pchip(Tw, Rw)(xx) * wh
+            Zx = scipy.interpolate.pchip(Tw, Zw)(xx) * wh
+            Rm = np.dot(Rx.T, expmt)
+            Zm = np.dot(Zx.T, expmt)
+
+            # compute (dR/dTw,dZ/dTw) via Fourier harmonics
+            expmt = np.exp(1j * mm * Tw[:, np.newaxis])
+            dR = 1j * np.dot(mm * Rm, expmt.T)
+            dZ = 1j * np.dot(mm * Zm, expmt.T)
+            # res = [dR(:)  dZ(:)]
+            dR = np.squeeze(dR.real)
+            dZ = np.squeeze(dZ.real)
+
+        else:
+            # another way to compute dR,dZ
+            dRT = np.diff(Rw) / np.diff(Tw)
+            x = (Tw[1:] + Tw[0:-1]) * 0.5
+            x = np.concatenate([[x[-1] - 2 * np.pi], x, [x[0] + 2 * np.pi]])
+            dRT = np.concatenate([[dRT[-1]], dRT, [dRT[0]]])
+            dR = scipy.interpolate.pchip(x, dRT)(Tw)
+            dZT = np.diff(Zw) / np.diff(Tw)
+            dZT = np.concatenate([[dZT[-1]], dZT, [dZT[0]]])
+            dZ = scipy.interpolate.pchip(x, dZT)(Tw)
+
+        # compute physical Bn & Bt from Bwr & Bwz
+        nA = np.sqrt(dR**2 + dZ**2)
+        Bn = (Bwr * dZ - Bwz * dR) / nA
+        Bt = (Bwr * dR + Bwz * dZ) / nA
+
+        # Fourier decompose Bn, Bt & Bphi along Tw angle
+        expmt = np.exp(-1j * xx * mm) / (2 * np.pi)
+
+        Bnx = scipy.interpolate.pchip(Tw, Bn.values)(xx) * wh
+        Btx = scipy.interpolate.pchip(Tw, Bt.values)(xx) * wh
+        Bpx = scipy.interpolate.pchip(Tw, Bwphi.values)(xx) * wh
+        Bnm = np.dot(Bnx.T, expmt).T
+        Btm = np.dot(Btx.T, expmt).T
+        Bpm = np.dot(Bpx.T, expmt).T
+
+        # Store results
+        self['BPLASMA']['Bn_s'] = xarray.DataArray(Bn, dims=['Tw'], coords={'Tw': Tw})
+        self['BPLASMA']['Bt_s'] = xarray.DataArray(Bt, dims=['Tw'], coords={'Tw': Tw})
+        self['BPLASMA']['Bphi_s'] = xarray.DataArray(Bwphi, dims=['Tw'], coords={'Tw': Tw})
+        self['BPLASMA']['Bnm_s'] = xarray.DataArray(np.squeeze(Bnm), dims=['Mm'], coords={'Mm': self['BPLASMA']['Mm']})
+        self['BPLASMA']['Btm_s'] = xarray.DataArray(np.squeeze(Btm), dims=['Mm'], coords={'Mm': self['BPLASMA']['Mm']})
+        self['BPLASMA']['Bpm_s'] = xarray.DataArray(np.squeeze(Bpm), dims=['Mm'], coords={'Mm': self['BPLASMA']['Mm']})
+
+        return Bnm, Btm, Bpm
+
     @dynaLoad
     def load(self):
         self[self.sim] = xarray.Dataset()
@@ -1093,7 +1273,8 @@ class OMFITmars(SortedDict, OMFITobject):
 
     def plot_JPLASMA(self, Mmax=5, fig=None):
         """
-        Plot perturbed current components J1,J2,J3
+        Plot perturbed current components j1,j2,j3
+        The MARS variable "J*j" is shown, not the physical quantity "j"
         :param fig: specify target figure
         """
         from matplotlib import pyplot
@@ -1101,28 +1282,25 @@ class OMFITmars(SortedDict, OMFITobject):
 
         if fig is None:
             fig = pyplot.figure()
-        else:
-            pyplot.figure(num=fig.number)
-
         ax = None
         label = 'JPLASMA'
         MM = self[label]['Mm'].values
         labelstring = [str(m) for m in MM]
         for k, c in enumerate(['JM1', 'JM2', 'JM3']):
-            ax = ax1 = pyplot.subplot(3, 2, 2 * k + 1, sharex=ax)
+            ax = ax1 = fig.add_subplot(3, 2, 2 * k + 1, sharex=ax)
             ax1.plot(self[label]['s'], np.real(self[label][c]))
             for i, line in enumerate(ax1.lines):
                 line.set_label(labelstring[i])
             lines1 = ax1.get_lines()
             labelLines(lines1[np.abs(np.min(MM)) + 1 : np.abs(np.min(MM)) + Mmax + 1], xvals=(0.0, 1.0), align=False)
-            ax1.set_title('Re($J^%d_m$)' % (k + 1), y=0.8)
-            ax2 = pyplot.subplot(3, 2, 2 * (k + 1), sharex=ax)
+            ax1.set_title('Re($J\cdot j^%d_m$)' % (k + 1), y=0.8)
+            ax2 = fig.add_subplot(3, 2, 2 * (k + 1), sharex=ax)
             ax2.plot(self[label]['s'], np.imag(self[label][c]))
             for i, line in enumerate(ax2.lines):
                 line.set_label(labelstring[i])
             lines2 = ax2.get_lines()
             labelLines(lines2[np.abs(np.min(MM)) + 1 : np.abs(np.min(MM)) + Mmax + 1], xvals=(0.0, 1.0), align=False)
-            ax2.set_title('Im($J^%d_m$)' % (k + 1), y=0.8)
+            ax2.set_title('Im($J\cdot j^%d_m$)' % (k + 1), y=0.8)
         ax1.set_xlabel('s')
         ax2.set_xlabel('s')
         fig.tight_layout()
@@ -1153,7 +1331,8 @@ class OMFITmars(SortedDict, OMFITobject):
 
     def plot_BPLASMA(self, Mmax=5, fig=None):
         """
-        Plot perturbed field components B1,B2,B3
+        Plot perturbed field components b1,b2,b3
+        The MARS variable "J*b" is shown, not the physical quantity "b"
         :param Mmax: upper poloidal harmonic for labeling
         :param fig: specify target figure
         """
@@ -1162,28 +1341,25 @@ class OMFITmars(SortedDict, OMFITobject):
 
         if fig is None:
             fig = pyplot.figure()
-        else:
-            pyplot.figure(num=fig.number)
-
         ax = None
         label = 'BPLASMA'
         MM = self[label]['Mm'].values
         labelstring = [str(m) for m in MM]
         for k, c in enumerate(['BM1', 'BM2', 'BM3']):
-            ax = ax1 = pyplot.subplot(3, 2, 2 * k + 1, sharex=ax)
+            ax = ax1 = fig.add_subplot(3, 2, 2 * k + 1, sharex=ax)
             ax1.plot(self[label]['s'], np.real(self[label][c]))
             for i, line in enumerate(ax1.lines):
                 line.set_label(labelstring[i])
             lines1 = ax1.get_lines()
             labelLines(lines1[np.abs(np.min(MM)) + 1 : np.abs(np.min(MM)) + Mmax + 1], xvals=(0.0, 1.0), align=False)
-            ax1.set_title('Re($B^%d_m$)' % (k + 1), y=0.8)
-            ax2 = pyplot.subplot(3, 2, 2 * (k + 1), sharex=ax)
+            ax1.set_title('Re($J\cdot b^%d_m$)' % (k + 1), y=0.8)
+            ax2 = fig.add_subplot(3, 2, 2 * (k + 1), sharex=ax)
             ax2.plot(self[label]['s'], np.imag(self[label][c]))
             for i, line in enumerate(ax2.lines):
                 line.set_label(labelstring[i])
             lines2 = ax2.get_lines()
             labelLines(lines2[np.abs(np.min(MM)) + 1 : np.abs(np.min(MM)) + Mmax + 1], xvals=(0.0, 1.0), align=False)
-            ax2.set_title('Im($B^%d_m$)' % (k + 1), y=0.8)
+            ax2.set_title('Im($J\cdot b^%d_m$)' % (k + 1), y=0.8)
         ax1.set_xlabel('s')
         ax2.set_xlabel('s')
         fig.tight_layout()
@@ -1301,10 +1477,11 @@ class OMFITmars(SortedDict, OMFITobject):
         ax.set_ylabel('Z [m]')
         fig.tight_layout()
 
-    def plot_BrzSurf(self, II=False, fig=None):
+    def plot_BrzSurf(self, II=False, rsurf=False, fig=None):
         """
         Plot Br Bz (cyl. coords.) along specified surface
         :param II: plot on II radial grid index (default is plasma boundary)
+        :param rsurf: normalized radius of surface
         :param fig: specify target figure
         """
         # Pick last s-grid point for Bn plotting
@@ -1317,6 +1494,9 @@ class OMFITmars(SortedDict, OMFITobject):
 
         if II:
             Ns = II
+        elif rsurf:
+            # Find surface index
+            Ns = np.argmin(np.abs(self[self.sim]['s'].values - rsurf))
         else:
             Ns = self[self.sim]['Ns1'].values
         R = (self[self.sim]['R'].values) * self[self.sim]['R0EXP'].values
@@ -1336,21 +1516,22 @@ class OMFITmars(SortedDict, OMFITobject):
         BZ = self[label]['Bz'].values
         ax1 = pyplot.subplot(2, 2, 1)
         ax1.plot(Tw, np.real(BR[Ns, JJ]), linewidth=2)
-        ax1.set_ylabel('Re($\delta B_R$) [Gauss]')
+        ax1.set_ylabel('Re($\delta B_R$)')
 
         ax2 = pyplot.subplot(2, 2, 2)
         ax2.plot(Tw, np.imag(BR[Ns, JJ]), linewidth=2)
-        ax2.set_ylabel('Im($\delta B_R$) [Gauss]')
+        ax2.set_ylabel('Im($\delta B_R$)')
 
         ax3 = pyplot.subplot(2, 2, 3)
         ax3.plot(Tw, np.real(BZ[Ns, JJ]), linewidth=2)
-        ax3.set_ylabel('Re($\delta B_Z$) [Gauss]')
+        ax3.set_ylabel('Re($\delta B_Z$)')
         ax3.set_xlabel('GEOM angle')
 
         ax4 = pyplot.subplot(2, 2, 4)
         ax4.plot(Tw, np.imag(BZ[Ns, JJ]), linewidth=2)
-        ax4.set_ylabel('Im($\delta B_Z$) [Gauss]')
+        ax4.set_ylabel('Im($\delta B_Z$)')
         ax4.set_xlabel('GEOM angle')
+        fig.suptitle(r'$B_r$, $B_z$ vs poloidal angle at s(%s)=%s' % (Ns, self[self.sim]['s'][Ns].values))
         fig.tight_layout()
 
     def plot_BrzAbs(self, fig=None):
@@ -1371,7 +1552,7 @@ class OMFITmars(SortedDict, OMFITobject):
         label = 'BPLASMA'
         AmpB = np.sqrt(abs(self[label]['Br'].values) ** 2 + abs(self[label]['Bz'].values) ** 2 + abs(self[label]['Bphi'].values) ** 2)
         ax.plot(chi * 180 / np.pi, AmpB[Ns, :], linewidth=2)
-        ax.set_ylabel('$|\delta B|$ [Gauss/kAt]')
+        ax.set_ylabel('$|\delta B|$')
         ax.set_xlabel('Poloidal angle $[\circ]$')
         fig.tight_layout()
 
@@ -1401,7 +1582,33 @@ class OMFITmars(SortedDict, OMFITobject):
         pc = ax.pcolormesh(R, Z, AmpB, cmap='jet', antialiased=True)
         ax.set_aspect('equal', adjustable='box')
         fig.colorbar(pc, ax=ax)
-        ax.set_title('$|\delta B|$ [Gauss/kAt]')
+        ax.set_title('$|\delta B|$')
+        ax.set_xlabel('R [m]')
+        ax.set_ylabel('Z [m]')
+        fig.tight_layout()
+
+    def plot_XrzMap(self, fig=None):
+        """
+        Plot |X| (R,Z) map inside plasma
+        :param fig: specify target figure
+        """
+        from matplotlib import pyplot
+
+        if fig is None:
+            fig = pyplot.figure()
+        Ns = self[self.sim]['Ns1'].values
+        label = 'XPLASMA'
+        ax = fig.add_subplot(1, 1, 1)
+        Xr = self[label]['Xr'].isel(sp=range(Ns)).values.real
+        # Xz = self[label]['Xz'].isel(sp=range(Ns)).values
+        # Xphi = self[label]['Xphi'].isel(sp=range(Ns)).values
+        # AmpX = np.sqrt(abs(Xr) ** 2 + abs(Xz) ** 2 + abs(Xphi) ** 2) * self[self.sim]['R0EXP'].values * 1.0e3
+        R = self[self.sim]['R'].isel(s=range(Ns)) * self[self.sim]['R0EXP']
+        Z = self[self.sim]['Z'].isel(s=range(Ns)) * self[self.sim]['R0EXP']
+        pc = ax.pcolormesh(R, Z, Xr, cmap='jet', antialiased=True)
+        ax.set_aspect('equal', adjustable='box')
+        fig.colorbar(pc, ax=ax)
+        ax.set_title('Radial displacement $X_r$')
         ax.set_xlabel('R [m]')
         ax.set_ylabel('Z [m]')
         fig.tight_layout()
@@ -1473,6 +1680,226 @@ class OMFITmars(SortedDict, OMFITobject):
         ax.legend(loc=0)
         fig.tight_layout()
 
+    def plot_BntpSurf(self, fig=None):
+        """
+        Plot Bn, Bt, Bphi on surface calculated by get_BatSurf
+        :param fig: specify target figure
+        """
+        from matplotlib import pyplot
+
+        if fig is None:
+            fig = pyplot.figure()
+        Tw = self['BPLASMA']['Tw']
+        ax1 = fig.add_subplot(3, 2, 1)
+        ax1.plot(Tw * 180 / np.pi, np.real(self['BPLASMA']['Bn_s']), 'b-')
+        ax1.set_ylabel('Re(Bn)')
+
+        ax2 = fig.add_subplot(3, 2, 2)
+        ax2.plot(Tw * 180 / np.pi, np.imag(self['BPLASMA']['Bn_s']), 'b-')
+        ax2.set_ylabel('Im(Bn)')
+
+        ax3 = fig.add_subplot(3, 2, 3)
+        ax3.plot(Tw * 180 / np.pi, np.real(self['BPLASMA']['Bt_s']), 'b-')
+        ax3.set_ylabel('Re(Bt)')
+
+        ax4 = fig.add_subplot(3, 2, 4)
+        ax4.plot(Tw * 180 / np.pi, np.imag(self['BPLASMA']['Bt_s']), 'b-')
+        ax4.set_ylabel('Im(Bt)')
+
+        ax5 = fig.add_subplot(3, 2, 5)
+        ax5.plot(Tw * 180 / np.pi, np.real(self['BPLASMA']['Bphi_s']), 'b-')
+        ax5.set_ylabel(r'Re($B_{\phi}$)')
+        ax5.set_xlabel(r'$\theta$ [$^{\circ}$]')
+
+        ax6 = fig.add_subplot(3, 2, 6)
+        ax6.plot(Tw * 180 / np.pi, np.imag(self['BPLASMA']['Bphi_s']), 'b-')
+        ax6.set_ylabel(r'Im($B_{\phi}$)')
+        ax6.set_xlabel(r'$\theta$ [$^{\circ}$]')
+        fig.tight_layout()
+
+    def plot_BntpSurfMm(self, fig=None):
+        """
+        Plot poloidal Fourier harmonics of Bn, Bt, Bphi on surface calculated by get_BatSurf
+        :param fig: specify target figure
+        """
+        from matplotlib import pyplot
+
+        if fig is None:
+            fig = pyplot.figure()
+        Tw = self['BPLASMA']['Tw']
+        ax1 = fig.add_subplot(3, 2, 1)
+        ax1.plot(self['BPLASMA']['Mm'], np.real(self['BPLASMA']['Bnm_s']), 'bo')
+        ax1.set_ylabel(r'$Re(B^m_n)$')
+        ax1.grid()
+
+        ax2 = fig.add_subplot(3, 2, 2)
+        ax2.plot(self['BPLASMA']['Mm'], np.imag(self['BPLASMA']['Bnm_s']), 'bo')
+        ax2.set_ylabel(r'$Im(B^m_n)$')
+        ax2.grid()
+
+        ax3 = fig.add_subplot(3, 2, 3)
+        ax3.plot(self['BPLASMA']['Mm'], np.real(self['BPLASMA']['Btm_s']), 'bo')
+        ax3.set_ylabel(r'$Re(B^m_t)$')
+        ax3.grid()
+
+        ax4 = fig.add_subplot(3, 2, 4)
+        ax4.plot(self['BPLASMA']['Mm'], np.imag(self['BPLASMA']['Btm_s']), 'bo')
+        ax4.set_ylabel(r'$Im(B^m_t)$')
+        ax4.grid()
+
+        ax5 = fig.add_subplot(3, 2, 5)
+        ax5.plot(self['BPLASMA']['Mm'], np.real(self['BPLASMA']['Bpm_s']), 'bo')
+        ax5.set_ylabel(r'Re($B^m_{\phi}$)')
+        ax5.set_xlabel('m')
+        ax5.grid()
+
+        ax6 = fig.add_subplot(3, 2, 6)
+        ax6.plot(self['BPLASMA']['Mm'], np.imag(self['BPLASMA']['Bpm_s']), 'bo')
+        ax6.set_ylabel(r'Im($B^m_{\phi}$)')
+        ax6.set_xlabel('m')
+        ax6.grid()
+        fig.tight_layout()
+
+    def plot_BatSurf3D(self, fig=None, rsurf=1.0, ntor=-1):
+        """
+        Plot Bn on surface calculated by get_BatSurf, normalized and reconstructed in (chi,phi) real space
+        :param fig: specify target figure
+        :param rsurf: normalized surface radius for plotting, default is plasma boundary
+        :param n: toroidal mode number for inverse Fourier transorm in toroidal angle
+        """
+        from matplotlib import pyplot
+        from matplotlib.colors import BoundaryNorm
+        from matplotlib.ticker import MaxNLocator
+
+        if fig is None:
+            fig = pyplot.figure()
+        try:
+            Bn = self['BPLASMA']['Bn_s']
+        except KeyError:
+            raise ('ERROR: calculate B on surface before plotting')
+        Tw = self['BPLASMA']['Tw']
+        phi = np.linspace(0, 2 * np.pi, self[self.sim]['Nchi'].values)
+        expnphi = np.exp(1j * ntor * phi)
+        Bna = np.real(np.dot(expnphi[:, np.newaxis], Bn.values[np.newaxis, :])).T / np.max(np.abs(Bn.values))
+        [pp, cc] = np.meshgrid(phi, Tw)
+        pp = pp * 180 / np.pi
+        cc = cc * 180 / np.pi
+        # Find surface index
+        IIsurf = np.argmin(np.abs(self[self.sim]['s'].values - rsurf))
+
+        levels = MaxNLocator(nbins=25).tick_values(Bna.min(), Bna.max())
+        cmap = pyplot.get_cmap('jet')
+        norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
+        ax0 = pyplot.gca()
+        cf = ax0.contourf(pp, cc, Bna, levels=levels, cmap=cmap)
+        cf2 = ax0.contour(cf, levels=cf.levels[::2], colors='k')
+        cbar = fig.colorbar(cf, ax=ax0)
+        cbar.ax.set_ylabel(r'[A.U.]')
+        ax0.set_title(r'$B_{n}$ n=%s - at r/a = %s' % (ntor, str(self[self.sim]['s'][IIsurf].values)))
+        ax0.set_xlabel(r'$\Phi [^\circ]$')
+        ax0.set_ylabel(r'$\Theta [^\circ]$')
+
+        # Find index of HFS, LFS, TOP, BOT
+        k = self[self.sim]['R'].isel(s=IIsurf).argmin(dim='chi').values
+        chi_HFS = Tw[k] * 180 / np.pi
+        k = self[self.sim]['R'].isel(s=IIsurf).argmax(dim='chi').values
+        chi_LFS = Tw[k] * 180 / np.pi
+        k = self[self.sim]['Z'].isel(s=IIsurf).argmin(dim='chi').values
+        chi_bot = Tw[k] * 180 / np.pi
+        k = self[self.sim]['Z'].isel(s=IIsurf).argmax(dim='chi').values
+        chi_top = Tw[k] * 180 / np.pi
+
+        ax0.text(10, chi_HFS, 'HFS', bbox={'facecolor': 'white', 'alpha': 1.0, 'pad': 5})
+        ax0.text(10, chi_LFS, 'LFS', bbox={'facecolor': 'white', 'alpha': 1.0, 'pad': 5})
+        ax0.text(10, chi_top, 'TOP', bbox={'facecolor': 'white', 'alpha': 1.0, 'pad': 5})
+        ax0.text(10, chi_bot, 'BOT', bbox={'facecolor': 'white', 'alpha': 1.0, 'pad': 5})
+
+        pyplot.tight_layout()
+
+    def plot_Bwarp(self, fig=None, rsurf=1.0):
+        """
+        Visualize B-field perturbation on specific surface
+        :param fig: specify target figure
+        :param rsurf: normalized surface radius for plotting, default is plasma boundary
+        """
+        from matplotlib import pyplot
+
+        if fig is None:
+            fig = pyplot.figure()
+        if 'Br' not in self['BPLASMA']:
+            self.get_Bcyl()
+
+        # Find surface index
+        IIsurf = np.argmin(np.abs(self[self.sim]['s'].values - rsurf))
+
+        Rw = self[self.sim]['R'].isel(s=IIsurf)
+        Zw = self[self.sim]['Z'].isel(s=IIsurf)
+
+        try:
+            Bwr = self['BPLASMA']['Br'].isel(s=IIsurf)
+            Bwz = self['BPLASMA']['Bz'].isel(s=IIsurf)
+        except IndexError:
+            print('ERROR: Selected index is out of BPLASMA calculation domain')
+            return
+        B0 = self[self.sim]['B0EXP'].values
+        R0 = self[self.sim]['R0EXP'].values
+        h = B0 / self[self.sim]['Ns1'].values
+        Bwr_r = np.real(Bwr)
+        Bwz_r = np.real(Bwz)
+        Bt = np.sqrt(Bwr_r**2 + Bwz_r**2)
+        Bt[np.where(Bt == 0.0)] = 1.0
+        R1 = Rw + h * Bwr_r / Bt
+        Z1 = Zw + h * Bwz_r / Bt
+        R2 = np.vstack((Rw, R1))
+        Z2 = np.vstack((Zw, Z1))
+        pyplot.plot(Rw * R0, Zw * R0, 'c-')
+        pyplot.plot(R2 * R0, Z2 * R0, 'b-')
+        pyplot.axis('equal')
+        pyplot.xlabel('R [m]')
+        pyplot.ylabel('Z [m]')
+
+    def plot_Xwarp(self, fig=None, rsurf=1.0):
+        """
+        Visualize displacement perturbation on specific surface
+        :param fig: specify target figure
+        :param rsurf: normalized surface radius for plotting, default is plasma boundary
+        """
+        from matplotlib import pyplot
+
+        if fig is None:
+            fig = pyplot.figure()
+        if 'Xr' not in self['XPLASMA']:
+            self.get_Xcyl()
+
+        # Find wall surface index
+        IIsurf = np.argmin(np.abs(self[self.sim]['s'].values - rsurf))
+
+        Rw = self[self.sim]['R'].isel(s=IIsurf)
+        Zw = self[self.sim]['Z'].isel(s=IIsurf)
+
+        try:
+            Xwr = self['XPLASMA']['Xr'].isel(sp=IIsurf)
+            Xwz = self['XPLASMA']['Xz'].isel(sp=IIsurf)
+        except IndexError:
+            print('ERROR: Selected index is out of BPLASMA calculation domain')
+            return
+        B0 = self[self.sim]['B0EXP'].values
+        R0 = self[self.sim]['R0EXP'].values
+        h = R0 / self[self.sim]['Ns1'].values
+        Xwr_r = np.real(Xwr)
+        Xwz_r = np.real(Xwz)
+        Xt = np.sqrt(Xwr_r**2 + Xwz_r**2)
+        Xt[np.where(Xt == 0.0)] = 1.0
+        R1 = Rw + h * Xwr_r / Xt
+        Z1 = Zw + h * Xwz_r / Xt
+        R2 = np.vstack((Rw, R1))
+        Z2 = np.vstack((Zw, Z1))
+        pyplot.plot(Rw * R0, Zw * R0, 'c-')
+        pyplot.plot(R2 * R0, Z2 * R0, 'b-')
+        pyplot.axis('equal')
+        pyplot.xlabel('R [m]')
+        pyplot.ylabel('Z [m]')
+
 
 class OMFITnColumns(SortedDict, OMFITascii):
     r"""
