@@ -22,6 +22,8 @@ from omfit_classes.utils_fusion import is_device
 
 import omas
 import numpy as np
+from scipy.interpolate import interp1d
+
 
 __all__ = ['NoFitException', 'OMFITosborneProfile', 'OMFITpFile']
 
@@ -404,6 +406,7 @@ class OMFITpFile(SortedDict, OMFITascii):
         SortedDict.__init__(self)
 
         self.legacy = False
+        self.is_remapped = False  # define whether this is the original pFile or its psin grid has been remapped
 
         self.descriptions = SortedDict()
         self.descriptions['ne'] = 'Electron density'
