@@ -255,9 +255,9 @@ class OMFITprofiles(OMFITncDataset):
                             profs['grid.rho_tor_norm'] = fit.sel(time=time).values
                     elif q == 'psi_n':
                         if fit_coordinate == 'psi_n':
-                            profs['grid.rho_pol_norm'] = derived['psi_n'].values
+                            profs['grid.rho_pol_norm'] = np.sqrt(derived['psi_n'].values)
                         else:
-                            profs['grid.rho_pol_norm'] = fit.sel(time=time).values
+                            profs['grid.rho_pol_norm'] = np.sqrt(fit.sel(time=time).values)
                     elif q == 'n_e':
                         profs['electrons.density_thermal'] = fit.sel(time=time).values
                     elif q == 'T_e':
