@@ -56,52 +56,81 @@ __all__ = [
 ]
 
 
-standard_test_keywords = dict(
-    run_tests=True,
-    allow_gui=bool(('rootGUI' in OMFITaux) and OMFITaux['rootGUI']),
-    allow_long=True,
-    failfast=False,
-    force_gh_status=None,
-    force_gh_comment=None,
-    only_these=None,
-    there_can_be_only_one=True,
-)
+# Make a class for standard_test_keywords so it can have a docstring that describes the keywords
+class StandardTestKeywords(dict):
+    """
+    Set of standard keywords to be accepted by defaultVars in OMFIT regression tests
 
-# Documentation for standard test keywords
-standard_test_keywords_doc = '''
-:param run_tests: bool
-    Switch for enabling execution of tests. Disabling execution allows importing of objects from the test script.
+    Provided as a central feature to make developing new tests easier.
 
-:param allow_gui: bool
-    Allows test units that require GUI interactions to execute. Recommended: decorate affected units with
-    @unittest.skipUnless(allow_gui, 'GUIs not allowed')
-    def test_blah_blah(...
+    :param run_tests: bool
+        Switch for enabling execution of tests. Disabling execution allows
+        importing of objects from the test script.
 
-:param allow_long: bool
-    Allows long test units to run. What qualifies as long is subjective. Recommended: decorate with
-    @unittest.skipUnless(allow_long, 'Long tests are not allowed')
-    def test_blah(...
+    :param allow_gui: bool
+        Allows test units that require GUI interactions to execute. Recommended:
+        decorate affected units with:
+            @unittest.skipUnless(allow_gui, 'GUIs not allowed')
+            def test_blah_blah(...
 
-:param failfast: bool
-    Halt execution at the first failure instead of testing remaining units. Passed to unittest.
+    :param allow_long: bool
+        Allows long test units to run. What qualifies as long is subjective.
+        Recommended: decorate with:
+            @unittest.skipUnless(allow_long, 'Long tests are not allowed')
+            def test_blah(...
 
-:param force_gh_status: bool
-    If not None, override default GitHub status posting behavior and force posting or not posting
+    :param failfast: bool
+        Halt execution at the first failure instead of testing remaining units.
+        Passed to unittest.
 
-:param force_gh_comment: int
-    If not None, override default GitHub comment posting behavior with this value.
-    The value is interpreted as bits, so 5 means 1 and 4.
-        1 (bit 0): do a post or update
-        2 (bit 1): if there's a failure to report, do a post or update
-        4 (bit 2): edit an existing comment instead of posting if possible, otherwise post
-        8 (bit 3): edit the top comment to add or update the test report (ignored if not combined with bit 2)
+    :param force_gh_status: bool [optional]
+        If not None, override default GitHub status posting behavior and force posting or not posting.
 
-:param only_these: string or list of strings
-    Names of test units to run (with or without leading `test_`). Other test units will not be run. (None to run all tests)
+    :param force_gh_comment: int [optional]
+        If not None, override default GitHub comment posting behavior with this value.
+        The value is interpreted as bits, so 5 means 1 and 4.
+            1 (bit 0): do a post or update
+            2 (bit 1): if there's a failure to report, do a post or update
+            4 (bit 2): edit an existing comment instead of posting if possible, otherwise post
+            8 (bit 3): edit the top comment to add or update the test report (ignored if not combined with bit 2)
 
-:param there_can_be_only_one: int or bool
-    Set test report comment cleanup behavior; not very relevant if bit 3 of gh_comment isn't set.
-'''
+    :param only_these: string or list of strings [optional]
+        Names of test units to run (with or without leading `test_`).
+        Other test units will not be run. (None to run all tests)
+
+    :param there_can_be_only_one: int or bool
+        Set test report comment cleanup behavior; not very relevant if bit 3 of
+        gh_comment isn't set. This value is interpreted as a set of binary flags,
+        so 6 should be interpreted as options 2 and 4 being active.
+
+        A value of True is converted into 255 (all the bits are True, including unused bits).
+        A value of None is replaced by the default value, which is True or 255.
+        A float or string will work if it can be converted safely by int().
+
+        1: Any of the flags will activate this feature. The 1 bit has no special meaning beyond activation.
+            If active, old github comments will be deleted. The newest report may be retained.
+        2: Limit deletion to reports that match the combined context of the test being run.
+        4: Only protect the latest comment if it reports a failure; if the last test passed, all comments may be deleted
+        8: Limit scope to comments with matching username
+
+    """
+
+    def __init__(self):
+        self.update(
+            dict(
+                run_tests=True,
+                allow_gui=bool(('rootGUI' in OMFITaux) and OMFITaux['rootGUI']),
+                allow_long=True,
+                failfast=False,
+                force_gh_status=None,
+                force_gh_comment=None,
+                only_these=None,
+                there_can_be_only_one=True,
+            )
+        )
+
+
+standard_test_keywords = StandardTestKeywords()
 
 auto_comment_marker = (
     '<!--This comment was automatically generated by OMFIT and was not posted directly by a human. ' 'A2ZZJfxk2910x2AZZf -->'
@@ -225,10 +254,13 @@ class setup_warnings(object):
             warnings.filterwarnings('ignore', category=FutureWarning, message='The Panel class is removed*')
             warnings.filterwarnings('ignore', category=PendingDeprecationWarning)
             warnings.filterwarnings('ignore', category=UserWarning, message="Warning: converting a masked element to nan.")
+            warnings.filterwarnings('ignore', category=DeprecationWarning, message='distutils Version classes are deprecated')
             # psycopg2
             warnings.filterwarnings('ignore', message='The psycopg2 wheel package will be renamed from release 2.8*')
             # pandas
             warnings.filterwarnings('ignore', message="Pandas doesn't allow columns to be created via a new attribute name*")
+            # omas
+            warnings.filterwarnings('ignore', message='omas cython failed*')
 
         if level <= 1:
             # Divide by 0
@@ -848,19 +880,9 @@ class OMFITtest(unittest.TestCase):
         overall context. The overall status will be posted if individual status
         reports are set to 2 (only on failure).
 
-    :param notify_slack: bool
-        Post automatic test report on slack
-
     :param notify_email: bool
         Automatically send a test report to the user via email
 
-    :param slack_channel: string
-        Slack's code for the channel where the slack report should be posted,
-        like "CHLS03MQC".
-
-    :param slack_username: string
-        The name to use for the apparent author of the slack post
-
     :param save_stats_to: dict-like
         Container for catching test statistics
 
@@ -893,10 +915,7 @@ class OMFITtest(unittest.TestCase):
     notify_gh_comment = 14 * auto_test_mode  # 14 is append report of failures only to top comment
     gh_individual_status = 2 + auto_test_mode  # 2 is only on failure, 3 is on failure even if overall status is off.
     gh_overall_status_after_individual = True
-    notify_slack = False
     notify_email = False
-    slack_channel = 'auto_test_reports'
-    slack_username = 'OMFIT_test_reports'
     save_stats_to = None
     stats_key = None
     topics_skipped = []
@@ -1331,7 +1350,7 @@ def test_comp_info_summary(short=False, long_form=False):
 
     :param long_form: bool
         Add more info to the end, concerning commit hashes. This is intended for
-        non-GitHub uses like email and slack, where the report won't be attached
+        non-GitHub uses like email, where the report won't be attached
         to a commit history in an obvious way. This triggers a simple append
         operation that will work even if short=True.
 
@@ -1371,7 +1390,6 @@ def manage_tests(
     force_gh_status=None,
     force_gh_comment=None,
     force_email=None,
-    force_slack=None,
     force_warning_level=None,
     print_report=True,
     there_can_be_only_one=True,
@@ -1419,10 +1437,6 @@ def manage_tests(
         None: notify_email on/off defined by test.
         True or False: force email notifications to be on or off.
 
-    :param force_slack: bool [optional]
-        None: notify_slack on/off defined by test.
-        True or False: force slack notifications to be on or off.
-
     :param force_warning_level: int [optional]
         None: warning_level defined by test.
         int: Force the warning level for all tests to be this value.
@@ -1494,7 +1508,6 @@ def manage_tests(
     ]
     gh_comment = [force_gh_comment or test.notify_gh_comment for test in tests]
     notify_email = [force_email or test.notify_email for test in tests]
-    notify_slack = [force_slack or test.notify_slack for test in tests]
     warning_levels = [force_warning_level or test.warning_level for test in tests]
     if there_can_be_only_one is True or there_can_be_only_one is None:
         # True turns on all the bits, even unused ones for future compatibility.
@@ -1507,10 +1520,6 @@ def manage_tests(
         test.force_gh_status = force_gh_status
 
     # Look up more settings from the tests
-    if np.any(notify_slack):
-        slack_channel = common_in_list([test.slack_channel for test in tests])
-    else:
-        slack_channel = None
     skipped_topics = [test.topics_skipped for test in tests]
     table_sorting_columns = common_in_list(['\n'.join(test.table_sorting_columns) for test in tests]).split('\n')
 
@@ -1597,7 +1606,6 @@ def manage_tests(
             new_skip += ['.'.join([context, skip]) for skip in skipped]
         skipped_topics = [new_skip]  # [[context+'.'+skipped for context, skipped in zip(contexts, skipped_topics)]]
         notify_email = [np.any(notify_email)]
-        notify_slack = [np.any(notify_slack)]
         results_tables = []
         for suite_test in suite_tests:
             meta_data.update(suite_test.meta_data)
@@ -1650,7 +1658,6 @@ def manage_tests(
     # Prepare reports
     the_gh_comment = ''
     email_body = ''
-    slack_message = ''
     errors = 0
     total_tests_run = 0
     total_tests_skipped = 0
@@ -1743,14 +1750,6 @@ def manage_tests(
                     email_part += '\n{}\n\n{}\n\nMeta data for {}:\n{}\n'.format(ft, er, ft, mdc)
             email_body += email_part + '-----'
 
-        # Handle slack message
-        if notify_slack[i]:
-            slack_message += conclusion + '\n' + test_count_report + '\n' + skipped_topics_report + '\n'
-            if len(failed_tests):
-                slack_message += 'Exception reports for {}:\n'.format(context)
-                for ft, er, mdc in zip(failed_long_names, error_reports, meta_data_chunks):
-                    slack_message += '{}\n{}\nMeta data for {}:\n{}\n'.format(ft, er, ft, mdc)
-
         # Print to console
         if print_report:
             if state:
@@ -1813,39 +1812,6 @@ def manage_tests(
         printd('Sending email: subject = {}, body = {}'.format(email_subject, email_body), topic='omfit_testing')
         send_email(to=MainSettings['SETUP']['email'], subject=email_subject, message=email_body, fromm=MainSettings['SETUP']['email'])
 
-    # Finalize and post slack message (if applicable)
-    if len(slack_message):
-        from omfit_classes.omfit_slack import post_to_slack
-
-        remote, org, repository, branch = get_gh_remote_org_repo_branch()
-        prn = get_pull_request_number(org=org, repository=repository)
-        if prn is not None:
-            pr_info = 'PR URL: https://github.com/{org:}/{rep:}/pull/{prn:}\n'.format(prn=prn, rep=repository, org=org)
-        else:
-            pr_info = ''
-        slack_message = '''
-{notice:}
-{errors:} errors while testing {context:}
-{run:}
-{body:}
-{comp:}
-{pr_info:}'''.strip().format(
-            notice=notice,
-            errors=errors,
-            context=' + '.join(contexts),
-            body=slack_message,
-            comp=test_comp_info_summary(),
-            pr_info=pr_info,
-            run=total_test_count_report,
-        )
-        if results_table is not None:
-            slack_message += '\n\n' + rtf
-        try:
-            post_to_slack(message=slack_message, channel=slack_channel)
-        except ValueError as excp:  # Bad/missing token (expected common problem) results in ValueError
-            printe(repr(excp))
-            printe('Slack post failed')
-
     # Closing remarks to console
     err_count = '\n{er:} error(s) were detected while testing {ct:}'.format(er=errors, ct=' + '.join(contexts))
     error_function = printe if errors > 0 else printi
