@@ -1085,11 +1085,11 @@ def intensity_sat0(
     notcond = (alpha_quench != 0) + (np.abs(kx0_e) == 0)
     # Calculate intensity_given using scalar TGLF calibration coefficients
     cnorm = c0 * pols
-    cnorm = cnorm / ((ks ** etg_fact) * (ks > 1) + 1 * (ks <= 1))
-    intensity_out = cnorm * (wd0 ** 2) * (gnet ** exp1 + c1 * gnet) / (kp ** 4)
-    intensity_out = intensity_out / (((1 + cx_cy * kx0_e ** 2) ** 2) * cond + 1 * notcond)
+    cnorm = cnorm / ((ks**etg_fact) * (ks > 1) + 1 * (ks <= 1))
+    intensity_out = cnorm * (wd0**2) * (gnet**exp1 + c1 * gnet) / (kp**4)
+    intensity_out = intensity_out / (((1 + cx_cy * kx0_e**2) ** 2) * cond + 1 * notcond)
     intensity_out = intensity_out / (((1 + (alpha_x * kx0_e) ** 4) ** 2) * cond + 1 * notcond)
-    intensity_out = intensity_out / B_unit ** 2
+    intensity_out = intensity_out / B_unit**2
     intensity_out = np.nan_to_num(intensity_out)
     intensity_out[
         intensity_out == 0
@@ -1098,32 +1098,375 @@ def intensity_sat0(
     # See example with default parameters below and regression test 'omfit/regression/test_tglf_satpy.py'
 
 
-def intensity_sat1(
+def get_zonal_mixing(
+    ky_mix,
+    gamma_mix,
+    **kw,
+):
+    '''
+    :param ky_mix: poloidal wavenumber [nk]
+
+    :param gamma_mix: most unstable growth rates [nk]
+
+    :param **kw: keyword list in input.tglf
+    '''
+
+    nky = len(ky_mix)
+    gammamax1 = gamma_mix[0]
+    kymax1 = ky_mix[0]
+    testmax1 = gammamax1 / kymax1
+    testmax2 = 0
+    jmax1 = 0
+    jmax2 = 0
+
+    j1 = 0
+    j2 = 0
+
+    kycut = (
+        0.8 * abs(kw['ZS_2']) / np.sqrt(kw['TAUS_2'] * kw['MASS_2'])
+    )  # ITG/ETG-scale separation (for TEM scales see [Creely et al., PPCF, 2019])
+    kyhigh = 0.15 * abs(kw['ZS_1']) / np.sqrt(kw['TAUS_1'] * kw['MASS_1'])
+
+    for j in range(1, nky):
+        ky0 = ky_mix[j]
+        if ky0 < kycut:
+            j1 = j1 + 1
+        if ky0 < kyhigh:
+            j2 = j2 + 1
+        test = gamma_mix[j] / ky0
+        if ky0 < kycut:
+            if test > testmax1:
+                testmax1 = test
+                kymax1 = ky0
+                jmax1 = j
+        if ky0 > kycut:
+            if test > testmax2:
+                testmax2 = test
+                kymax2 = ky0
+                jmax2 = j
+    # handle exceptions
+    if j1 == nky - 1:
+        j1 = nky - 2
+    if jmax2 == 0:
+        jmax2 = j2
+
+    gammamax2 = gamma_mix[jmax2]
+    kymax2 = ky_mix[jmax2]
+    gammamax1 = gamma_mix[jmax1]
+    kymax1 = ky_mix[jmax1]
+    vzf1 = gammamax1 / kymax1
+    vzf2 = gammamax2 / kymax2
+
+    # Routine for better determination of gamma/ky peak
+    if jmax1 > 0 and jmax1 < j1:
+        f0 = gamma_mix[jmax1 - 1] / ky_mix[jmax1 - 1]
+        f1 = gamma_mix[jmax1] / ky_mix[jmax1]
+        f2 = gamma_mix[jmax1 + 1] / ky_mix[jmax1 + 1]
+        dky = ky_mix[jmax1 + 1] - ky_mix[jmax1 - 1]
+        x0 = (ky_mix[jmax1] - ky_mix[jmax1 - 1]) / dky
+        a = f0
+        x02 = x0**2
+        b = (f1 - f0 * (1 - x02) - f2 * x02) / (x0 - x02)
+        c = f2 - f0 - b
+        xmax = -b / (2.0 * c)
+        if xmax > 1.0:
+            kymax1 = ky_mix[jmax1 + 1]
+            gammamax1 = f2 * kymax1
+        elif xmax < 0.0:
+            kymax1 = ky_mix[jmax1 - 1]
+            gammamax1 = f0 * kymax1
+        else:
+            kymax1 = ky_mix[jmax1 - 1] + dky * xmax
+            gammamax1 = (a + b * xmax + c * xmax**2) * kymax1
+    vzf_mix = gammamax1 / kymax1
+    kymax_mix = kymax1
+    jmax_mix = jmax1
+    return vzf_mix, kymax_mix, jmax_mix
+
+
+def get_sat_params(sat_rule_in, ky, gammas, mts=5.0, ms=128, small=0.00000001, **kw):
+    '''
+    This function calculates the scalar saturation parameters and spectral shift needed
+    for the TGLF saturation rules, dependent on changes to 'tglf_geometry.f90' by Gary Staebler
+
+    :mts: the number of points in the s-grid (flux surface contour)
+    :ms: number of points along the arclength
+    :ds: the arc length differential on a flux surface
+    :R(ms): the major radius on the s-grid
+    :Z(ms): the vertical coordinate on the s-grid
+    :Bp(ms): the poloidal magnetic field on the s-grid normalized to B_unit
+    :**kw: input.tglf
+    '''
+    drmajdx_loc = kw['DRMAJDX_LOC']
+    drmindx_loc = kw['DRMINDX_LOC']
+    kappa_loc = kw['KAPPA_LOC']
+    s_kappa_loc = kw['S_KAPPA_LOC']
+    rmin_loc = kw['RMIN_LOC']
+    rmaj_loc = kw['RMAJ_LOC']
+    zeta_loc = kw['ZETA_LOC']
+    q_s = kw['Q_LOC']
+    q_prime_s = kw['Q_PRIME_LOC']
+    p_prime_s = kw['P_PRIME_LOC']
+    delta_loc = kw['DELTA_LOC']
+    s_delta_loc = kw['S_DELTA_LOC']
+    s_zeta_loc = kw['S_ZETA_LOC']
+    alpha_e_in = kw['ALPHA_E']
+    vexb_shear = kw['VEXB_SHEAR']
+    sign_IT = kw['SIGN_IT']
+    sign_Bt_in = kw['SIGN_BT']
+    units = kw['UNITS']
+    mass_2 = kw['MASS_2']
+    taus_2 = kw['TAUS_2']
+    zs_2 = kw['ZS_2']
+    alpha_ZF_in = kw['ALPHA_ZF']
+
+    zmaj_loc = 0.0
+    dzmajdx_loc = 0.0
+    norm_ave = 0.0
+    SAT_geo1_out = 0.0
+    SAT_geo2_out = 0.0
+    dlp = 0.0
+
+    R = np.zeros(ms + 1)
+    Z = np.zeros(ms + 1)
+    Bp = np.zeros(ms + 1)
+    Bt = np.zeros(ms + 1)
+    B = np.zeros(ms + 1)
+    b_geo = np.zeros(ms + 1)
+    qrat_geo = np.zeros(ms + 1)
+    sin_u = np.zeros(ms + 1)
+    s_p = np.zeros(ms + 1)
+    r_curv = np.zeros(ms + 1)
+    psi_x = np.zeros(ms + 1)
+    costheta_geo = np.zeros(ms + 1)
+
+    pi_2 = 2 * np.pi
+    if rmin_loc < 0.00001:
+        rmin_loc = 0.00001
+    vs_2 = np.sqrt(taus_2 / mass_2)
+    gamma_reference_kx0 = gammas[0, :]
+
+    # Miller geo
+    rmin_s = rmin_loc
+    Rmaj_s = rmaj_loc
+
+    # compute the arclength around the flux surface:
+    # initial values define dtheta
+    theta = 0.0
+    x_delta = np.arcsin(delta_loc)
+    arg_r = theta + x_delta * np.sin(theta)
+    darg_r = 1.0 + x_delta * np.cos(theta)
+    arg_z = theta + zeta_loc * np.sin(2.0 * theta)
+    darg_z = 1.0 + zeta_loc * 2.0 * np.cos(2.0 * theta)
+    r_t = -rmin_loc * np.sin(arg_r) * darg_r
+    z_t = kappa_loc * rmin_loc * np.cos(arg_z) * darg_z
+    l_t = np.sqrt(r_t**2 + z_t**2)
+
+    # scale dtheta by l_t to keep mts points in each ds interval of size pi_2/ms
+    dtheta = pi_2 / (mts * ms * l_t)
+    l_t1 = l_t
+    arclength = 0.0
+
+    while theta < pi_2:
+        theta = theta + dtheta
+        if theta > pi_2:
+            theta = theta - dtheta
+            dtheta = pi_2 - theta
+            theta = pi_2
+
+        arg_r = theta + x_delta * np.sin(theta)
+        darg_r = 1.0 + x_delta * np.cos(theta)  # d(arg_r)/dtheta
+        r_t = -rmin_loc * np.sin(arg_r) * darg_r  # dR/dtheta
+        arg_z = theta + zeta_loc * np.sin(2.0 * theta)
+        darg_z = 1.0 + zeta_loc * 2.0 * np.cos(2.0 * theta)  # d(arg_z)/dtheta
+        z_t = kappa_loc * rmin_loc * np.cos(arg_z) * darg_z  # dZ/dtheta
+        l_t = np.sqrt(r_t**2 + z_t**2)  # dl/dtheta
+        arclength = arclength + 0.50 * (l_t + l_t1) * dtheta  # arclength along flux surface in poloidal direction
+        l_t1 = l_t
+
+    # Find the theta points which map to an equally spaced s-grid of ms points along the arclength
+    # going clockwise from the outboard midplane around the flux surface
+    # by searching for the theta where dR**2 + dZ**2 >= ds**2 for a centered difference df=f(m+1)-f(m-1).
+    # This keeps the finite difference error of dR/ds, dZ/ds on the s-grid small
+    ds = arclength / ms
+    t_s = np.zeros(ms + 1)
+    t_s[ms] = -pi_2
+
+    # Make a first guess based on theta = 0.0
+    theta = 0.0
+    arg_r = theta + x_delta * np.sin(theta)
+    darg_r = 1.0 + x_delta * np.cos(theta)
+    arg_z = theta + zeta_loc * np.sin(2.0 * theta)
+    darg_z = 1.0 + zeta_loc * 2.0 * np.cos(2.0 * theta)
+    r_t = -rmin_loc * np.sin(arg_r) * darg_r
+    z_t = kappa_loc * rmin_loc * np.cos(arg_z) * darg_z
+    l_t = np.sqrt(r_t**2 + z_t**2)
+    dtheta = -ds / l_t
+    theta = dtheta
+    l_t1 = l_t
+
+    for m in range(1, int(ms / 2) + 1):
+        arg_r = theta + x_delta * np.sin(theta)
+        darg_r = 1.0 + x_delta * np.cos(theta)
+        arg_z = theta + zeta_loc * np.sin(2.0 * theta)
+        darg_z = 1.0 + zeta_loc * 2.0 * np.cos(2.0 * theta)
+        r_t = -rmin_loc * np.sin(arg_r) * darg_r
+        z_t = kappa_loc * rmin_loc * np.cos(arg_z) * darg_z
+        l_t = np.sqrt(r_t**2 + z_t**2)
+        dtheta = -ds / (0.5 * (l_t + l_t1))
+        t_s[m] = t_s[m - 1] + dtheta
+        theta = t_s[m] + dtheta
+        l_t1 = l_t
+
+    # distribute endpoint error over interior points
+    dtheta = (t_s[int(ms / 2)] - (-np.pi)) / (ms / 2)
+
+    for m in range(1, int(ms / 2) + 1):
+        t_s[m] = t_s[m] - (m) * dtheta
+        t_s[ms - m] = -pi_2 - t_s[m]
+
+    for m in range(0, ms + 1):
+        theta = t_s[m]
+        arg_r = theta + x_delta * np.sin(theta)
+        darg_r = 1.0 + x_delta * np.cos(theta)
+        arg_z = theta + zeta_loc * np.sin(2.0 * theta)
+        darg_z = 1.0 + zeta_loc * 2.0 * np.cos(2.0 * theta)
+
+        R[m] = rmaj_loc + rmin_loc * np.cos(arg_r)  # = R(theta)
+        Z[m] = zmaj_loc + kappa_loc * rmin_loc * np.sin(arg_z)  # = Z(theta)
+
+        R_t = -rmin_loc * np.sin(arg_r) * darg_r  # = dR/dtheta
+        Z_t = kappa_loc * rmin_loc * np.cos(arg_z) * darg_z  # = dZ/dtheta
+
+        l_t = np.sqrt(R_t**2 + Z_t**2)  # = dl/dtheta
+
+        R_r = (
+            drmajdx_loc + drmindx_loc * np.cos(arg_r) - np.sin(arg_r) * s_delta_loc * np.sin(theta) / np.sqrt(1.0 - delta_loc**2)
+        )  # = dR/dr
+        Z_r = (
+            dzmajdx_loc
+            + kappa_loc * np.sin(arg_z) * (drmindx_loc + s_kappa_loc)
+            + kappa_loc * np.cos(arg_z) * s_zeta_loc * np.sin(2.0 * theta)
+        )
+
+        det = R_r * Z_t - R_t * Z_r  # Jacobian
+        grad_r = abs(l_t / det)
+        if m == 0:
+            B_unit = 1.0 / grad_r  # B_unit choosen to make qrat_geo(0)/b_geo(0)=1.0
+            if drmindx_loc == 1.0:
+                B_unit = 1.0  # Waltz-Miller convention
+
+        Bp[m] = (rmin_s / (q_s * R[m])) * grad_r * B_unit
+        p_prime_s = p_prime_s * B_unit
+        q_prime_s = q_prime_s / B_unit
+        psi_x[m] = R[m] * Bp[m]
+
+    delta_s = 12.0 * ds
+    ds2 = 12.0 * ds**2
+    for m in range(0, ms + 1):
+        m1 = (ms + m - 2) % ms
+        m2 = (ms + m - 1) % ms
+        m3 = (m + 1) % ms
+        m4 = (m + 2) % ms
+        R_s = (R[m1] - 8.0 * R[m2] + 8.0 * R[m3] - R[m4]) / delta_s
+        Z_s = (Z[m1] - 8.0 * Z[m2] + 8.0 * Z[m3] - Z[m4]) / delta_s
+        s_p[m] = np.sqrt(R_s**2 + Z_s**2)
+        R_ss = (-R[m1] + 16.0 * R[m2] - 30.0 * R[m] + 16.0 * R[m3] - R[m4]) / ds2
+        Z_ss = (-Z[m1] + 16.0 * Z[m2] - 30.0 * Z[m] + 16.0 * Z[m3] - Z[m4]) / ds2
+        r_curv[m] = (s_p[m] ** 3) / (R_s * Z_ss - Z_s * R_ss)
+        sin_u[m] = -Z_s / s_p[m]
+
+    # Compute f=R*Bt such that the eikonal S which solves
+    # B*Grad(S)=0 has the correct quasi-periodicity S(s+Ls)=S(s)-2*pi*q_s, where Ls = arclength
+    f = 0.0
+    for m in range(1, ms + 1):
+        f = f + 0.5 * ds * (s_p[m - 1] / (R[m - 1] * psi_x[m - 1]) + s_p[m] / (R[m] * psi_x[m]))
+    f = pi_2 * q_s / f
+
+    for m in range(0, ms + 1):
+        Bt[m] = f / R[m]
+        B[m] = np.sqrt(Bt[m] ** 2 + Bp[m] ** 2)
+        qrat_geo[m] = (rmin_s / R[m]) * (B[m] / Bp[m]) / q_s
+        b_geo[m] = B[m]
+        costheta_geo[m] = -Rmaj_s * (Bp[m] / (B[m]) ** 2) * (Bp[m] / r_curv[m] - (f**2 / (Bp[m] * (R[m]) ** 3)) * sin_u[m])
+
+    qrat_geo0 = (rmin_s / R[0]) * (B[0] / Bp[0]) / q_s
+    b_geo0 = B[0]
+
+    for m in range(1, ms + 1):
+        dlp = s_p[m] * ds * (0.5 / Bp[m] + 0.5 / Bp[m - 1])
+        norm_ave += dlp
+        SAT_geo1_out += dlp * ((b_geo[0] / b_geo[m - 1]) ** 4 + (b_geo[0] / b_geo[m]) ** 4) / 2.0
+        SAT_geo2_out += dlp * ((qrat_geo[0] / qrat_geo[m - 1]) ** 4 + (qrat_geo[0] / qrat_geo[m]) ** 4) / 2.0
+
+    SAT_geo1_out = SAT_geo1_out / norm_ave
+    SAT_geo2_out = SAT_geo2_out / norm_ave
+    R_unit = Rmaj_s * b_geo[0] / (qrat_geo[0] * costheta_geo[0])
+    B_geo0_out = b_geo[0]
+    Bt0_out = f / Rmaj_s
+    grad_r0_out = b_geo[0] / qrat_geo[0]
+
+    # Compute spetral shift kx0_e
+    vexb_shear_s = vexb_shear * sign_IT
+    vexb_shear_kx0 = alpha_e_in * vexb_shear_s
+
+    wd0 = abs(ky / Rmaj_s)
+    kx0_factor = abs(b_geo[0] / qrat_geo[0] ** 2)
+
+    if alpha_ZF_in < 0.0:
+        kx0_factor = 1.0
+    kx0_factor = 1.0 + 0.40 * (kx0_factor - 1.0) ** 2
+
+    kyi = ky * vs_2 * mass_2 / abs(zs_2)
+    wE = kx0_factor * array([min(x / 0.3, 1.0) for x in kyi]) * vexb_shear_kx0 / gamma_reference_kx0
+    kx0_e = -(0.36 * vexb_shear_kx0 / gamma_reference_kx0 + 0.38 * wE * np.tanh((0.69 * wE) ** 6))
+
+    if sat_rule_in == 1:
+        kx0_e = -(0.53 * vexb_shear_kx0 / gamma_reference_kx0 + 0.25 * wE * np.tanh((0.69 * wE) ** 6))
+    elif sat_rule_in == 2:
+        vzf_out, kymax_out, jmax_out = get_zonal_mixing(ky, gamma_reference_kx0, **kw)
+        if abs(kymax_out * vzf_out * vexb_shear_kx0) > small:
+            kx0_e = -0.32 * ((ky / kymax_out) ** 0.3) * vexb_shear_kx0 / (ky * vzf_out)
+        else:
+            kx0_e = np.zeros(len(ky))
+    a0 = 1.3
+    if sat_rule_in == 1:
+        a0 = 1.45
+    elif sat_rule_in == 2:
+        a0 = 1.6
+    kx0_e = array([min(abs(x), a0) * x / abs(x) for x in kx0_e])
+    kx0_e[np.isnan(kx0_e)] = 0
+
+    return kx0_e, SAT_geo1_out, SAT_geo2_out, R_unit, Bt0_out, B_geo0_out, grad_r0_out
+
+
+def intensity_sat(
+    sat_rule_in,
     ky_spect,
     gp,
     kx0_e,
-    taus_2,
+    nmodes,
     expsub=2.0,
     alpha_zf_in=1.0,
     kx_geo0_out=1.0,
     SAT_geo_out=1.0,
     bz1=0.0,
     bz2=0.0,
-    zs_1=-1.0,
-    zs_2=1.0,
-    taus_1=1.0,
-    mass_1=0.000272313,
-    mass_2=1.0,
+    **kw,
 ):
     '''
     TGLF SAT1 from [Staebler et al., 2016, PoP], takes both GYRO and TGLF outputs as inputs
 
+    :param sat_rule_in: saturation rule [1, 2]
+
     :param ky_spect: poloidal wavenumber [nk]
 
     :param gp: growth rates [nk, nm]
 
     :param kx0_e: spectral shift of the radial wavenumber due to VEXB_SHEAR [nk]
 
+    :param nmodes_in: number of modes stored in quasi-linear weights [1, ..., 5]
+
     :param expsub: scalar exponent in gammaeff calculation [2.0]
 
     :param alpha_zf_in: scalar switch for the zonal flow coupling coefficient [1.0]
@@ -1136,107 +1479,108 @@ def intensity_sat1(
 
     :param bz2: scalar correction to zonal flow mixing term [0.0]
 
-    :param zs_1: scalar ratio of electron charge to the first ion charge (very likely to be -1.0)
-
-    :param zs_2: scalar ratio fo the first ion charge with itself [1.0]
-
-    :param taus_1: scalar ratio of electron temperature to itself [1.0]
-
-    :param taus_2: scalar ratio of first ion temperature to electron temperature
-
-    :param mass_1: scalar ratio of electron to first ion [0.000272313]
-
-    :param mass_2: scalar ratio of first ion mass to itself [1.0]
-
+    :param **kw: keyword list in input.tglf
     '''
 
-    nmodes_in = len(gp[0, :])
     nky = len(ky_spect)
-    gammas1 = gp[:, 0]  # SAT1 uses the growth rates of the most unstable modes
+    if len(np.shape(gp)) > 1:
+        gammas1 = gp[:, 0]  # SAT1 and SAT2 use the growth rates of the most unstable modes
+    else:
+        gammas1 = gp
+    gamma_net = np.zeros(nky)
 
-    etg_streamer = 1.05
-    kyetg = etg_streamer * abs(zs_2) / np.sqrt(taus_2 * mass_2)
+    if sat_rule_in == 1:
+        etg_streamer = 1.05
+        kyetg = etg_streamer * abs(kw['ZS_2']) / np.sqrt(kw['TAUS_2'] * kw['MASS_2'])
+        measure = np.sqrt(kw['TAUS_1'] * kw['MASS_2'])
 
     czf = abs(alpha_zf_in)
     small = 1.0e-10
     cz1 = 0.48 * czf
     cz2 = 1.0 * czf
-    ax = 1.15
-    ay = 0.56
-
-    gamma_net = gammas1 / (1.0 + (ax * kx_geo0_out * kx0_e) ** 4)
-
     cky = 3.0
     sqcky = np.sqrt(cky)
     cnorm = 14.29
 
-    kycut = 0.8 * abs(zs_2) / np.sqrt(taus_2 * mass_2)  # ITG/ETG-scale separation (for TEM scales see [Creely et al., PPCF, 2019])
-    kyhigh = 0.15 / np.sqrt(taus_1 * mass_1)
+    kycut = (
+        0.8 * abs(kw['ZS_2']) / np.sqrt(kw['TAUS_2'] * kw['MASS_2'])
+    )  # ITG/ETG-scale separation (for TEM scales see [Creely et al., PPCF, 2019])
+    kyhigh = 0.15 * abs(kw['ZS_1']) / np.sqrt(kw['TAUS_1'] * kw['MASS_1'])
 
-    gammamax1 = gamma_net[0]
-    kymax1 = ky_spect[0]
-    testmax1 = gammamax1 / kymax1
-    testmax2 = 0
-    jmax1 = 0
-    jmax2 = 0
+    vzf_out, kymax_out, jmax_out = get_zonal_mixing(ky_spect, gammas1, **kw)
 
-    j1 = 0
-    j2 = 0
+    if kw['RLNP_CUTOFF'] > 0.0:
+        ptot = 0
+        dlnpdr = 0
+        for i in range(1, kw['NS'] + 1, 1):
+            ptot += kw['AS_%s' % i] * kw['TAUS_%s' % i]  # only kinetic species
+            dlnpdr += kw['AS_%s' % i] * kw['TAUS_%s' % i] * (kw['RLNS_%s' % i] + kw['RLTS_%s' % i])
+        dlnpdr = kw['RMAJ_LOC'] * dlnpdr / max(ptot, 0.01)
 
-    for j in range(1, nky):
-        ky0 = ky_spect[j]
-        if ky0 < kycut:
-            j1 = j1 + 1
-        if ky0 < kyhigh:
-            j2 = j2 + 1
-        test = gamma_net[j] / ky0
-        if ky0 < kycut:
-            if test > testmax1:
-                testmax1 = test
-                jmax1 = j
-        if ky0 > kycut:
-            if test > testmax2:
-                testmax2 = test
-                jmax2 = j
-    # handle exceptions
-    if j1 == nky - 1:
-        j1 = nky - 2
-    if jmax2 == 0:
-        jmax2 = j2
+        if dlnpdr >= kw['RLNP_CUTOFF']:
+            dlnpdr = kw['RLNP_CUTOFF']
+        if dlnpdr < 4.0:
+            dlnpdr = 4.0
+    else:
+        dlnpdr = 12.0
 
-    gammamax2 = gamma_net[jmax2]
-    kymax2 = ky_spect[jmax2]
-    gammamax1 = gamma_net[jmax1]
-    kymax1 = ky_spect[jmax1]
+    if sat_rule_in == 2:
+        # SAT2 fit for CGYRO linear modes NF 2021 paper
+        b0 = 0.76
+        b1 = 1.22
+        b2 = 3.74
+        if nmodes > 1:
+            b2 = 3.55
+        b3 = 1.0
+        d1 = (kw['Bt0_out'] / kw['B_geo0_out']) ** 4  # PPCF paper 2020
+        d1 = d1 / kw['grad_r0_out']
+        Gq = kw['B_geo0_out'] / kw['grad_r0_out']
+        d2 = b3 / Gq**2
+        cnorm = b2 * (12.0 / dlnpdr)
+        kyetg = 1000.0  # does not impact SAT2
+        cky = 3.0
+        sqcky = np.sqrt(cky)
+        kycut = b0 * kymax_out
+        cz1 = 0.0
+        cz2 = 1.05 * czf
+        measure = 1.0 / kymax_out
 
-    # Routine for better determination of gamma/ky peak
-    if jmax1 > 0 and jmax1 < j1:
-        f0 = gamma_net[jmax1 - 1] / ky_spect[jmax1 - 1]
-        f1 = gamma_net[jmax1] / ky_spect[jmax1]
-        f2 = gamma_net[jmax1 + 1] / ky_spect[jmax1 + 1]
-        dky = ky_spect[jmax1 + 1] - ky_spect[jmax1 - 1]
-        x0 = (ky_spect[jmax1] - ky_spect[jmax1 - 1]) / dky
-        a = f0
-        x02 = x0 * x0
-        b = (f1 - f0 * (1 - x02) - f2 * x02) / (x0 - x02)
-        c = f2 - f0 - b
-        xmax = -b / (2.0 * c)
-        if xmax > 1.0:
-            kymax1 = ky_spect[jmax1 + 1]
-            gammamax1 = f2 * kymax1
-        elif xmax < 0.0:
-            kymax1 = ky_spect[jmax1 - 1]
-            gammamax1 = f0 * kymax1
-        else:
-            kymax1 = ky_spect[jmax1 - 1] + dky * xmax
-            gammamax1 = (a + b * xmax + c * xmax * xmax) * kymax1
+    ax = 0.0
+    ay = 0.0
+    exp_ax = 1
+    if kw['ALPHA_QUENCH'] == 0.0:
+        if sat_rule_in == 1:
+            # spectral shift model parameters
+            ax = 1.15
+            ay = 0.56
+            exp_ax = 4
+            units_in = []
+        elif sat_rule_in == 2:
+            ax = 1.21
+            ay = 1.0
+            exp_ax = 2
+            units_in = ['CGYRO']
 
-    vzf1 = gammamax1 / kymax1
-    vzf2 = gammamax2 / kymax2
-    dvzf = max(vzf2 - vzf1, 0.0)
-    dgamma = dvzf * kymax1
-    vzf3 = max(vzf1 - bz1 * dvzf, 0.0)
-    vzf4 = max(vzf1 - bz2 * dvzf, 0.0)
+    for j in range(0, nky):
+        kx = kx0_e[j]
+        if sat_rule_in == 2:
+            ky0 = ky_spect[j]
+            if ky0 < kycut:
+                kx_width = kycut / kw['grad_r0_out']
+            else:
+                kx_width = kycut / kw['grad_r0_out'] + b1 * (ky0 - kycut) * Gq
+            kx = kx * ky0 / kx_width
+        gamma_net[j] = gammas1[j] / (1.0 + abs(ax * kx) ** exp_ax)
+
+    if sat_rule_in == 1:
+        vzf_out, kymax_out, jmax_out = get_zonal_mixing(ky_spect, gamma_net, **kw)
+    else:
+        vzf_out = vzf_out * gamma_net[jmax_out] / max(gammas1[jmax_out], small)
+
+    gammamax1 = vzf_out * kymax_out
+    kymax1 = kymax_out
+    jmax1 = jmax_out
+    vzf1 = vzf_out
 
     # include zonal flow effects on growth rate model:
     gamma_mix1 = np.zeros(nky)
@@ -1244,10 +1588,17 @@ def intensity_sat1(
 
     for j in range(0, nky):
         gamma0 = gamma_net[j]
-        if ky_spect[j] < kymax1:
-            gamma[j] = max(gamma0 - cz1 * (kymax1 - ky_spect[j]) * vzf1, 0.0)
-        else:
-            gamma[j] = cz2 * gammamax1 + max(gamma0 - cz2 * vzf1 * ky_spect[j], 0.0)
+        ky0 = ky_spect[j]
+        if sat_rule_in == 1:
+            if ky0 < kymax1:
+                gamma[j] = max(gamma0 - cz1 * (kymax1 - ky0) * vzf1, 0.0)
+            else:
+                gamma[j] = cz2 * gammamax1 + max(gamma0 - cz2 * vzf1 * ky0, 0.0)
+        elif sat_rule_in == 2:
+            if ky0 < kymax1:
+                gamma[j] = gamma0
+            else:
+                gamma[j] = gammamax1 + max(gamma0 - cz2 * vzf1 * ky0, 0.0)
         gamma_mix1[j] = gamma[j]
 
     # Mix over ky>kymax with integration weight
@@ -1269,21 +1620,34 @@ def intensity_sat1(
         gamma_mix1[j] = gamma_ave / mixnorm1
 
     # intensity
-    nmodes_in = 2
-    field_spectrum_out = np.zeros((21, 2))
+    field_spectrum_out = np.zeros((nky, nmodes))
     for j in range(0, nky):
         gamma0 = gp[j, 0]
         ky0 = ky_spect[j]
         kx = kx0_e[j]
-        for i in range(0, nmodes_in):
+        if sat_rule_in == 1:
+            sat_geo_factor = kw['SAT_geo0_out']
+            kx_width = ky0
+        if sat_rule_in == 2:
+            if ky0 < kycut:
+                kx_width = kycut / kw['grad_r0_out']
+                sat_geo_factor = kw['SAT_geo0_out'] * d1 * kw['SAT_geo1_out']
+            else:
+                kx_width = kycut / kw['grad_r0_out'] + b1 * (ky0 - kycut) * Gq
+                sat_geo_factor = kw['SAT_geo0_out'] * (d1 * kw['SAT_geo1_out'] * kycut + (ky0 - kycut) * d2 * kw['SAT_geo2_out']) / ky0
+            kx = kx * ky0 / kx_width
+
+        for i in range(0, nmodes):
             gammaeff = 0.0
             if gamma0 > small:
                 gammaeff = gamma_mix1[j] * (gp[j, i] / gamma0) ** expsub
             if ky0 > kyetg:
                 gammaeff = gammaeff * np.sqrt(ky0 / kyetg)
-            field_spectrum_out[j, i] = SAT_geo_out * (cnorm * gammaeff * gammaeff / ky0 ** 4) / (1.0 + ay * (kx_geo0_out * kx) ** 2) ** 2
+            field_spectrum_out[j, i] = measure * cnorm * ((gammaeff / (kx_width * ky0)) / (1.0 + ay * kx**2)) ** 2
+            if 'GYRO' not in units_in:
+                field_spectrum_out[j, i] = sat_geo_factor * field_spectrum_out[j, i]
     phinorm = field_spectrum_out
-    return phinorm  # is the intensity
+    return phinorm  # SAT1 or SAT2 intensity
 
 
 def flux_integrals(
@@ -1395,7 +1759,6 @@ def sum_ky_spectrum(
     :param alpha_x: scalar TGLF SAT0 calibration coefficient [1.15] (from TGLF 2008 POP Eq.13)
 
     :param \**kw: any additional argument should follow the naming convention of the TGLF_inputs
-            for SAT1 the TAUS_2 argument is needed
 
     :return: dictionary with summations over ky spectrum:
             * particle_flux_integral: [nm, ns, nf]
@@ -1422,8 +1785,8 @@ def sum_ky_spectrum(
             * potential
             / intensity_sat0(ky_spect, gp, ave_p0, R_unit, kx0_e, 1.25, 32.48, 0.534, 1.547, 0.56, 1.15)
         )
-    elif sat_rule_in in [1.0, 1, 'SAT1']:
-        intensity_factor = intensity_sat1(ky_spect, gp, kx0_e, kw['TAUS_2'])
+    elif sat_rule_in in [1.0, 1, 'SAT1', 2.0, 2, 'SAT2']:
+        intensity_factor = intensity_sat(sat_rule_in, ky_spect, gp, kx0_e, NM, **kw)
     elif sat_rule_in in [-1.0, -1, 'DESAT']:
         intensity_factor = (
             intensity_desat(ky_spect, kw['P_PRIME_LOC'], kw['Q_LOC'], kw['TAUS_2'])
