@@ -1497,6 +1497,22 @@ class OMFITmaintree(OMFITproject):
 
         return projects
 
+    def showExecDiag(self):
+        """
+        display execution diagram
+        """
+
+        def printMe(tmp, depth):
+            for child in tmp:
+                filename = re.sub('__console__.py', '', os.path.split(child['filename'])[1])
+                printi('* ' * (depth + 1) + " %s %3.3fs [%s]" % (filename, child['time'], sizeof_fmt(child['memory'])))
+                printMe(child['child'], depth + 1)
+
+        printi('\nExecution diagram:\n------------------')
+        from omfit_classes.omfit_python import ExecDiagPtr
+
+        printMe(ExecDiagPtr, 0)
+
 
 class OMFITmainscratch(OMFITtmp):
     def __init__(self):
