@@ -106,7 +106,10 @@ def _clipboard_get(self, selection=None, type=None):
     try:
         return self.clipboard_get(selection=selection, type=type)
     except tk.TclError:
-        return self.clipboard_get(type=type)
+        try:
+            return self.clipboard_get(type=type)
+        except tk.TclError:
+            return self.clipboard_get()
 
 
 def omfit_sel_handle(offset, length, selection=None, type=None):
@@ -139,7 +142,9 @@ def _paste(self):
         return
     clipboard = clipboard.replace('\r\n', '\n')
     clipboard = clipboard.replace('\r', '\n')
-    clipboard = clipboard.replace("\n", "\\n")
+
+    self.clipboard_clear()
+    self.clipboard_append(clipboard)
 
     # delete the selected text, if any
     try:
